#!/bin/bash

# Function to convert hexadecimal LSN to numeric value
hex_lsn_to_numeric() {
	echo "$1" | awk -F/ '{ printf "0x%s", $1$2 }'
}

# Function to calculate distance
calculate_distance() {
    local lsn1_numeric=$(hex_lsn_to_numeric "$1")
    local lsn2_numeric=$(hex_lsn_to_numeric "$2")
    echo $(( lsn1_numeric - lsn2_numeric ))
}

# Function to update replication_distance table
update_replication_distance() {
    psql -h "$1" -p "$2" -d "$3" -c \
    "INSERT INTO slot_lag (slot_name, lag, snapshot) VALUES ('$5', $4, $6)";
}

COUNTER=0
start=$SECONDS

while true; do
    primary_results=$(psql -h localhost -p 5442 -d postgres -c "SELECT slot_name, confirmed_flush_lsn FROM pg_replication_slots where slot_type = 'logical';" -t)
    standby_results=$(psql -h localhost -p 5443 -d postgres -c "SELECT slot_name, confirmed_flush_lsn FROM pg_replication_slots where slot_type = 'logical';" -t)

    let	COUNTER++

    IFS=$'\n'
    for primary_row in $primary_results; do
        primary_slot=$(echo "$primary_row" | cut -d '|' -f 1 | sed -e 's/^[[:space:]]*//' -e 's/[[:space:]]*$//')
        primary_lsn=$(echo "$primary_row" | cut -d '|' -f 2 | sed -e 's/^[[:space:]]*//' -e 's/[[:space:]]*$//')


        for standby_row in $standby_results; do
            standby_slot=$(echo "$standby_row" | cut -d '|' -f 1 | sed -e 's/^[[:space:]]*//' -e 's/[[:space:]]*$//')

            if [ "$standby_slot" = "$primary_slot" ]; then
                standby_lsn=$(echo "$standby_row" | cut -d '|' -f 2 | sed -e 's/^[[:space:]]*//' -e 's/[[:space:]]*$//')
		#echo "Primary LSN: $primary_lsn Primary Slot: $primary_slot Standby Slot: $standby_slot Standby LSN: $standby_lsn SnapshotCount:$COUNTER"
                distance=$(calculate_distance "$primary_lsn" "$standby_lsn")
            	if [ $distance -ne 0 ]; then
                	update_replication_distance "localhost" "5442" "postgres" "$distance" "$primary_slot" "$COUNTER"
		fi
            fi
        done
    done

    duration=$(( SECONDS - start ))
    if [ $duration -ge $1 ]; then
		echo "duration:$duration >= user supplied value:$1, thus exiting, current_snaphot: $COUNTER"
		exit
    fi

done
